(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     36751,       1307]*)
(*NotebookOutlinePosition[     37805,       1343]*)
(*  CellTagsIndexPosition[     37761,       1339]*)
(*WindowFrame->Normal*)



Notebook[{


Cell[TextData["Optimizing Expressions"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Using the Optimize.m package"], "Subtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Terry Robb\n<tdr@vaxc.cc.monash.edu.au>"], "Subsubtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"There is sometimes a need to improve, or otherwise optimize, the way in \
which some expressions are evaluated."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Consider for example a function definition like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["h[x_] := Erf[x] + Erf[x]^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " and the way the expression on the right is evaluated for a given \
numerical ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[". The ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Erf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" function will be evaluated ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["twice",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[" (",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Share[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " not-with-standing), which is quite unnecessary in this case. If however \
we had defined the function as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["h[x_] := Block[{e=Erf[x]}, e+e^2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", then ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Erf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " would be evaluated just once and that is a more optimized evaluation.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The package ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize.m",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" implements a command called ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" to automate the above type of expression optimization.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Needs[\"Optimize`\"];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["What ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" does is replace an expression with a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldBlock",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " object, which is a device for creating a block of code that will be held \
unevaluated. The idea is that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldBlock",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" will ultimately be used inside a function or a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Compile",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " definition etc, where the hold can be released and the optimized code \
inserted.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here are two simple examples which illustrate a typical use of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["f[x_Real] := Optimize[Sin[x] + Cos[Sin[x]]];\n?f"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Global`f\n\nf[x_Real] := Block[{O$1}, O$1 = Sin[x]; O$1 + Cos[O$1]]"], "Info",\

  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["and"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["g = Compile[x, Optimize[Sin[x] + Cos[Sin[x]]]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
CompiledFunction[{_Real}, {0, 0, 5, 0}, 
  {{1, 17}, {4, 1, 0}, {49, 0, 1}, {17, 1, 2}, {50, 2, 3}, 
   {33, 2, 3, 4}, {8, 4}}, 
  Function[{x}, Block[{O$1}, O$1 = Sin[x]; O$1 + Cos[O$1]]]]\
\>", 
"\<\
CompiledFunction[{x}, Block[{O$1}, 
 
   O$1 = Sin[x]; O$1 + Cos[O$1]], -CompiledCode-]\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox["In each case ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" evaluates itself (an ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Evaluate",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" is not required here, because  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " insists on that anyway in this context) and optimizes the expression ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sin[x] + Cos[Sin[x]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" so as to avoid the unnecessary double evaluation of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sin[x]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[". Thus when you call ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[2.2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["g[2.2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sin[2.2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" will be evaluated just ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["once",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[" and not twice.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that only basic ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["structural",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[
  " optimization is performed \[Dash] in other words only if two or more \
sub-expressions have ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["literally",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[
  " the same structural form will any optimization take place. No \
mathematical knowledge is therefore used in this procedure. See example 6 \
below for a simple way to effectively add some extra knowledge though.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Commands"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["There are three routines defined in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize.m",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and here's what they do:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["?Optimize"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Optimize[expr] optimizes an expression and returns a\n   HoldBlock object. \
An example usage is: Optimize[Sin[x] +\n   Cos[Sin[x]]] and it returns an \
object that can be used\n   when defining a function, for example: f[x_Real] \
:=\n   Optimize[Sin[x] + Cos[Sin[x]]]; or it can be used as an\n   argument \
inside Compile or Function as in: g =\n   Compile[x, Optimize[Sin[x] + \
Cos[Sin[x]]]];. An\n   optimized object can also be evaluated directly using\n\
   Normal or ReleaseHold as in: Normal[Optimize[Sin[x] +\n   Cos[Sin[x]]] /. \
x->2.2]."], "Info",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[CellGroupData[{Cell[TextData["?HoldBlock"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"HoldBlock[vars, body] represents an optimized expression.\n   The commands \
SetDelayed, Compile, Function, Normal and\n   ReleaseHold all know about \
HoldBlock and will convert it\n   to Block as necessary."], "Info",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[CellGroupData[{Cell[TextData["?Cost"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Cost[expr] estimates the cost of evaluating an expression\n   by simply \
counting the number of basic elementary\n   operations that are used. An \
example usage is:\n   Cost[Sin[x] + Cos[Sin[x]]] which returns Cos + Plus +\n \
  2*Sin. Cost also works with optimized objects, so if you\n   apply (Cost[#] \
- Cost[Optimize[#]])& to an expression\n   you will see the operations that \
might be saved."], "Info",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 1"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Here's a fairly long expression:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["expr = Nest[(#+Sin[#])&, x, 10];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Here's how we would define an optimized function to evaluate it:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["f1[x_Real]  :=  Optimize[expr];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"And here's how we would compile the optimized expression:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["f2 = Compile[x, Optimize[expr]];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Now let's use the un-optimized expression in a command to see how fast it \
is:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData[
"Plot[expr//Evaluate, {x, 0, 0.01},\n     DisplayFunction->Identity]; // \
Timing"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
{4.5*Second, Null}\
\>", 
"\<\
{4.5 Second, Null}\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["Now let's use the optimized expression:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData[
"Plot[f1[x], {x, 0, 0.01},\n     DisplayFunction->Identity]; // Timing"], 
  "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
{1.350000000000000006*Second, Null}\
\>", 
"\<\
{1.35 Second, Null}\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[
"And finally let's use the compiled optimized expression:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData[
"Plot[f2[x], {x, 0, 0.01},\n     DisplayFunction->Identity]; // Timing"], 
  "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
{0.25*Second, Null}\
\>", 
"\<\
{0.25 Second, Null}\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["In each case the timings get better."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["To see what optimization has taken place use:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["?f1"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Global`f1\n\nf1[x_Real] := \n  Block[{O$1, O$2, O$3, O$4, O$5, O$6, O$7, \
O$8, O$9, \n    O$10, O$11, O$12, O$13, O$14, O$15, O$16, O$17}, \n   O$1 = \
Sin[x]; O$2 = x + O$1; O$3 = Sin[O$2]; \n    O$4 = O$3 + O$2; O$5 = Sin[O$4]; \
O$6 = O$5 + O$4; \n    O$7 = Sin[O$6]; O$8 = O$7 + O$6; O$9 = Sin[O$8]; \n    \
O$10 = O$9 + O$8; O$11 = Sin[O$10]; O$12 = O$11 + O$10; \n    O$13 = \
Sin[O$12]; O$14 = O$13 + O$12; O$15 = Sin[O$14]; \n    O$16 = O$15 + O$14; \
O$17 = Sin[O$16]; \n    O$16 + O$17 + Sin[O$16 + O$17]]"], "Info",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["Note the cost of the original expression is:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[expr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
512*Plus + 1023*Sin\
\>", 
"\<\
512 Plus + 1023 Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["while the cost of the optimized expression is just:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[Optimize[expr]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
10*Plus + 17*Set + 10*Sin\
\>", 
"\<\
10 Plus + 17 Set + 10 Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox["The optimization has reduced the 1023 ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sin",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" evaluations down to just 10, by finding 17 rules.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 2"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Here's a big expression which we optimize:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"expr = Expand[(Sin[x*y] + Log[x] + Cosh[x^-y]^5)^10];\noexpr = \
Optimize[expr];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"The time to evaluate the expression at a typical point is:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["(expr        /. {x->1.1, y->0.5}) // Timing"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
{1.016666666666666607*Second, 1.025093642170576222*10^9}\
\>", 
"\<\
                           9
{1.01667 Second, 1.02509 10 }\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[
"But the optimized expression evaluates a little faster:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Normal[oexpr /. {x->1.1, y->0.5}] // Timing"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
{0.3500000000000000888*Second, 1.025093642170576222*10^9}\
\>", 
"\<\
                        9
{0.35 Second, 1.02509 10 }\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[
"The expression costs indicate why the latter is faster:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[expr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
55*Cosh + 55*Log + Plus + 200*Power + 55*Sin + 173*Times\
\>", 
"\<\
55 Cosh + 55 Log + Plus + 200 Power + 55 Sin + 173 Times\
\>"], "Output",\

  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[CellGroupData[{Cell[TextData["Cost[oexpr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
Cosh + Log + Plus + 29*Power + 31*Set + Sin + 65*Times\
\>", 
"\<\
Cosh + Log + Plus + 29 Power + 31 Set + Sin + 65 Times\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 3"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["You can use ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " to show structural information that is not otherwise obvious in the \
output from some commands such as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Solve",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["D",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Integrate",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" etc. For example:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData[
"Optimize[z /.\n       Solve[z^4 - 12*z^3 - 16*z^2 + 4*z + c == 0, z]\n       \
 ]//InputForm"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
InputForm[HoldBlock[{O$1, O$2, O$3, O$4, O$5, O$6, O$7, 
    O$8, O$9, O$10, O$11, O$12, O$13, O$14, O$15, O$16, 
    O$17, O$18, O$19, O$20, O$21, O$22, O$23, O$24, O$25, 
    O$26, O$27, O$28, O$29}, 
   O$1 = 2^(-1/3); O$2 = 2^(1/3); O$3 = 12*c; 
    O$4 = 5040*c; O$5 = 400 + O$3; O$6 = -14672 + O$4; 
    O$7 = O$5^3; O$8 = O$6^2; O$9 = -4*O$7; 
    O$10 = O$9 + O$8; O$11 = O$10^(1/2); 
    O$12 = O$11 + O$6; O$13 = O$12^(-1/3); 
    O$14 = O$12^(1/3); O$15 = -(O$2*O$5*O$13)/3; 
    O$16 = (O$2*O$5*O$13)/3; O$17 = -(O$1*O$14)/3; 
    O$18 = (O$1*O$14)/3; O$19 = 140/3 + O$16 + O$18; 
    O$20 = O$19^(-1/2); O$21 = O$19^(1/2); 
    O$22 = -616*O$20; O$23 = 616*O$20; O$24 = -O$21/2; 
    O$25 = O$21/2; O$26 = 280/3 + O$15 + O$17 + O$22; 
    O$27 = 280/3 + O$15 + O$17 + O$23; O$28 = O$26^(1/2); 
    O$29 = O$27^(1/2); {3 + O$24 - O$28/2, 
     3 + O$24 + O$28/2, 3 + O$25 - O$29/2, 
     3 + O$25 + O$29/2}]]\
\>", 
"\<\
HoldBlock[{O$1, O$2, O$3, O$4, O$5, O$6, O$7, O$8, O$9, 
   O$10, O$11, O$12, O$13, O$14, O$15, O$16, O$17, O$18, 
   O$19, O$20, O$21, O$22, O$23, O$24, O$25, O$26, O$27, 
   O$28, O$29}, O$1 = 2^(-1/3); O$2 = 2^(1/3); O$3 = 12*c; 
   O$4 = 5040*c; O$5 = 400 + O$3; O$6 = -14672 + O$4; 
   O$7 = O$5^3; O$8 = O$6^2; O$9 = -4*O$7; 
   O$10 = O$9 + O$8; O$11 = O$10^(1/2); O$12 = O$11 + O$6; 
   O$13 = O$12^(-1/3); O$14 = O$12^(1/3); 
   O$15 = -(O$2*O$5*O$13)/3; O$16 = (O$2*O$5*O$13)/3; 
   O$17 = -(O$1*O$14)/3; O$18 = (O$1*O$14)/3; 
   O$19 = 140/3 + O$16 + O$18; O$20 = O$19^(-1/2); 
   O$21 = O$19^(1/2); O$22 = -616*O$20; O$23 = 616*O$20; 
   O$24 = -O$21/2; O$25 = O$21/2; 
   O$26 = 280/3 + O$15 + O$17 + O$22; 
   O$27 = 280/3 + O$15 + O$17 + O$23; O$28 = O$26^(1/2); 
   O$29 = O$27^(1/2); {3 + O$24 - O$28/2, 
    3 + O$24 + O$28/2, 3 + O$25 - O$29/2, 
    3 + O$25 + O$29/2}]\
\>"], "Output",
  Evaluatable->False,
  LineSpacing->{0.6, 0},
  AspectRatioFixed->True]}, Open]]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 4"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The symbols ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["e",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["g",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" which are defined below, each optimize the expression ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sin[x] + Cos[Sin[x]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" in slightly different contexts:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["e = Optimize[Sin[x] + Cos[Sin[x]]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
HoldBlock[{O$1}, O$1 = Sin[x]; O$1 + Cos[O$1]]\
\>", 
"\<\
HoldBlock[{O$1}, O$1 = Sin[x]; O$1 + Cos[O$1]]\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["f[x_Real] := Optimize[Sin[x] + Cos[Sin[x]]]"], "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["g = Compile[x, Optimize[Sin[x] + Cos[Sin[x]]]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
CompiledFunction[{_Real}, {0, 0, 5, 0}, 
  {{1, 17}, {4, 1, 0}, {49, 0, 1}, {17, 1, 2}, 
   {50, 2, 3}, {33, 2, 3, 4}, {8, 4}}, 
  Function[{x}, Block[{O$1}, 
    O$1 = Sin[x]; O$1 + Cos[O$1]]]]\
\>", 
"\<\
CompiledFunction[{x}, Block[{O$1}, 
 
   O$1 = Sin[x]; O$1 + Cos[O$1]], -CompiledCode-]\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox["The three methods can be evaluated at the point say ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x->2.2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" as follows:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["{Normal[e /. x->2.2], f[2.2], g[2.2]}"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
{1.499083092380501356, 1.499083092380501356, 
  1.499083092380501356}\
\>", "\<\
{1.49908, 1.49908, 1.49908}\
\>"], "Output",\

  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["The evaluation cost of the un-optimized code is:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[Sin[x] + Cos[Sin[x]]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
Cos + Plus + 2*Sin\
\>", 
"\<\
Cos + Plus + 2 Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["While the evaluation cost of the optimized code is:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[Optimize[Sin[x] + Cos[Sin[x]]]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
Cos + Plus + Set + Sin\
\>", 
"\<\
Cos + Plus + Set + Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 5"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Big expressions like the following are a common application of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["expr = Sum[(Sin[x] + Cos[Sin[x]])^n, {n, 1, 100}];"], "Input",
  AspectRatioFixed->True],

Cell[TextData["The initial cost is large"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[expr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
100*Cos + 100*Plus + 99*Power + 200*Sin\
\>", 
"\<\
100 Cos + 100 Plus + 99 Power + 200 Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["But the optimized cost is much better"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Cost[Optimize[expr]]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
Cos + 2*Plus + 99*Power + 3*Set + Sin\
\>", 
"\<\
Cos + 2 Plus + 99 Power + 3 Set + Sin\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]]}, Open]],

Cell[CellGroupData[{Cell[TextData["Example 6"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Here's a simple expression:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["expr = (1+x*y)/(1-x*y)"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
(1 + x*y)/(1 - x*y)\
\>", "\<\
1 + x y
-------
1 - x y\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData["Let's try optimize it:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Optimize[expr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
(1 + x*y)/(1 - x*y)\
\>", "\<\
1 + x y
-------
1 - x y\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox["No optimization however takes place here because ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " only sees structural stuff, and so doesn't understand anything about the \
mathematics of multiplication. By looking at the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["FullForm",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" of the expression we see why ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" doesn't optimize the expression:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["FullForm[expr]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData["\<\
FullForm[(1 + x*y)/(1 - x*y)]\
\>", 
"\<\
Times[Power[Plus[1, Times[-1, x, y]], -1], 
 
  Plus[1, Times[x, y]]]\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox[
  "No structures are explicitly duplicated in the above, and so no \
optimization is performed. We can however tell ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " one fact about multiplication that will let it see more structure. By \
turning ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " into a right associative binary operator (rather than a flat orderless \
operator), extra structural information appears.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["To do this we repeatedly apply the rule ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times[a_,b__] :> TIMES[a,Times[b]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" to turn ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" into a right associative binary operator named ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["TIMES",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[". Then we call ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" on that new structure, and at the end replace ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["TIMES",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" with ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" to get back the normal ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" operator. Thus",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData[
"Optimize[expr //. Times[a_, b__] :> TIMES[a, Times[b]]\n        ] /. TIMES \
-> Times"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
HoldBlock[{O$1}, O$1 = x*y; (1 + O$1)/(1 - O$1)]\
\>", 
"\<\
                            1 + O$1
HoldBlock[{O$1}, O$1 = x y; -------]
                            1 - O$1\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox["The above has now extracted out the common ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x*y",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" term.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["To make the above easier a second argument named ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times->True",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" is provided with ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[", so that you can get the same effect by simply typing",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{Cell[TextData["Optimize[expr, Times->True]"], "Input",
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
HoldBlock[{O$1}, O$1 = x*y; (1 + O$1)/(1 - O$1)]\
\>", 
"\<\
                            1 + O$1
HoldBlock[{O$1}, O$1 = x y; -------]
                            1 - O$1\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[TextData[{
  StyleBox[
  "In order to save a small amount of initilization time, this feature has ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["not",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[" been implemented as a normal option however, and so ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Options[Optimize]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" is not modifiable.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]],

Cell[CellGroupData[{Cell[TextData["Caveats"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["There is one slight caveat with using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  ". It doesn't understand about expressions that contain anything with a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldAll",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" (or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldFirst",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldRest",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  ") attribute. Tweak the source code yourself if you desparately need to \
optimize such objects.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Also this implementation of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " works well if a smallish number (say less than 500) of the optimization \
variables (the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["O$",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " variables) are ultimately needed. The worst case is if the number of \
variables is near half the number of basic components in a very large \
expression, but such cases are rare. If efficiency were a major issue then ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Optimize",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" would be best implemented in C. The ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" version does however work well for the common cases.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]}, Open]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 640}, {0, 460}},
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{520, 365},
WindowMargins->{{28, Automatic}, {Automatic, 1}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
AVU/IFiQKFD000000Tn0D09?RA0000000PBJL0901kP0AP1Y06`0I@1^0642Ch1@
0Tn9<000000219Y@0T07^000000000000000009?P50000000000000000000000
00000000000000000000000000000000\>"
]



(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1711, 51, 97, 2, 70, "Title",
  Evaluatable->False],
Cell[1811, 55, 106, 2, 70, "Subtitle",
  Evaluatable->False],
Cell[1920, 59, 120, 2, 70, "Subsubtitle",
  Evaluatable->False],
Cell[2043, 63, 186, 4, 70, "Text",
  Evaluatable->False],
Cell[2232, 69, 1873, 66, 70, "Text",
  Evaluatable->False],
Cell[4108, 137, 606, 21, 70, "Text",
  Evaluatable->False],
Cell[4717, 160, 74, 1, 70, "Input"],
Cell[4794, 163, 1208, 42, 70, "Text",
  Evaluatable->False],
Cell[6005, 207, 401, 14, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[6429, 223, 101, 1, 70, "Input"],
Cell[6533, 226, 144, 4, 70, "Info",
  Evaluatable->False]
}, Open  ]],
Cell[6689, 232, 77, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[6789, 236, 99, 1, 70, "Input"],
Cell[6891, 239, 379, 12, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[7282, 253, 2152, 78, 70, "Text",
  Evaluatable->False],
Cell[9437, 333, 846, 25, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[10306, 360, 85, 2, 70, "Section",
  Evaluatable->False],
Cell[10394, 364, 377, 13, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[10794, 379, 62, 1, 70, "Input"],
Cell[10859, 382, 625, 10, 70, "Info",
  Evaluatable->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[11516, 394, 63, 1, 70, "Input"],
Cell[11582, 397, 283, 5, 70, "Info",
  Evaluatable->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[11897, 404, 58, 1, 70, "Input"],
Cell[11958, 407, 477, 8, 70, "Info",
  Evaluatable->False]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[12476, 417, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[12565, 421, 106, 2, 70, "Text",
  Evaluatable->False],
Cell[12674, 425, 85, 1, 70, "Input"],
Cell[12762, 428, 139, 3, 70, "Text",
  Evaluatable->False],
Cell[12904, 433, 84, 1, 70, "Input"],
Cell[12991, 436, 132, 3, 70, "Text",
  Evaluatable->False],
Cell[13126, 441, 85, 1, 70, "Input"],
Cell[13214, 444, 154, 4, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[13391, 450, 134, 3, 70, "Input"],
Cell[13528, 455, 139, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[13679, 464, 113, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[13815, 468, 126, 3, 70, "Input"],
Cell[13944, 473, 157, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[14113, 482, 131, 3, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[14267, 487, 126, 3, 70, "Input"],
Cell[14396, 492, 141, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[14549, 501, 110, 2, 70, "Text",
  Evaluatable->False],
Cell[14662, 505, 119, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[14804, 509, 56, 1, 70, "Input"],
Cell[14863, 512, 591, 9, 70, "Info",
  Evaluatable->False]
}, Open  ]],
Cell[15466, 523, 118, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[15607, 527, 63, 1, 70, "Input"],
Cell[15673, 530, 141, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[15826, 539, 125, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[15974, 543, 73, 1, 70, "Input"],
Cell[16050, 546, 153, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[16215, 555, 420, 14, 70, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[16667, 571, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[16756, 575, 116, 2, 70, "Text",
  Evaluatable->False],
Cell[16875, 579, 134, 3, 70, "Input"],
Cell[17012, 584, 133, 3, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[17168, 589, 96, 1, 70, "Input"],
Cell[17267, 592, 218, 9, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[17497, 603, 130, 3, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[17650, 608, 96, 1, 70, "Input"],
Cell[17749, 611, 213, 9, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[17974, 622, 130, 3, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[18127, 627, 63, 1, 70, "Input"],
Cell[18193, 630, 218, 9, 70, "Output",
  Evaluatable->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[18443, 641, 64, 1, 70, "Input"],
Cell[18510, 644, 212, 8, 70, "Output",
  Evaluatable->False]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[18763, 654, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[18852, 658, 1059, 40, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[19934, 700, 146, 3, 70, "Input"],
Cell[20083, 705, 1937, 44, 70, "Output",
  Evaluatable->False]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[22061, 751, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[22150, 755, 1019, 38, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[23192, 795, 87, 1, 70, "Input"],
Cell[23282, 798, 195, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[23489, 807, 96, 1, 70, "Input"],

Cell[CellGroupData[{
Cell[23608, 810, 99, 1, 70, "Input"],
Cell[23710, 813, 384, 13, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[24106, 828, 399, 14, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[24528, 844, 90, 1, 70, "Input"],
Cell[24621, 847, 200, 8, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[24833, 857, 122, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[24978, 861, 79, 1, 70, "Input"],
Cell[25060, 864, 139, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[25211, 873, 125, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[25359, 877, 89, 1, 70, "Input"],
Cell[25451, 880, 147, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[25639, 889, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[25728, 893, 332, 11, 70, "Text",
  Evaluatable->False],
Cell[26063, 906, 103, 1, 70, "Input"],
Cell[26169, 909, 99, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26291, 913, 63, 1, 70, "Input"],
Cell[26357, 916, 181, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[26550, 925, 111, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26684, 929, 73, 1, 70, "Input"],
Cell[26760, 932, 177, 7, 70, "Output",
  Evaluatable->False]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[26978, 941, 86, 2, 70, "Section",
  Evaluatable->False],
Cell[27067, 945, 101, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[27191, 949, 75, 1, 70, "Input"],
Cell[27269, 952, 144, 8, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[27425, 962, 96, 2, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[27544, 966, 67, 1, 70, "Input"],
Cell[27614, 969, 144, 8, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[27770, 979, 967, 32, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[28760, 1013, 67, 1, 70, "Input"],
Cell[28830, 1016, 201, 9, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[29043, 1027, 854, 28, 70, "Text",
  Evaluatable->False],
Cell[29900, 1057, 1722, 62, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[31645, 1121, 139, 3, 70, "Input"],
Cell[31787, 1126, 260, 10, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[32059, 1138, 381, 14, 70, "Text",
  Evaluatable->False],
Cell[32443, 1154, 656, 22, 70, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[33122, 1178, 80, 1, 70, "Input"],
Cell[33205, 1181, 260, 10, 70, "Output",
  Evaluatable->False]
}, Open  ]],
Cell[33477, 1193, 673, 22, 70, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[34182, 1217, 84, 2, 70, "Section",
  Evaluatable->False],
Cell[34269, 1221, 1141, 41, 70, "Text",
  Evaluatable->False],
Cell[35413, 1264, 1326, 42, 70, "Text",
  Evaluatable->False]
}, Closed]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)
